import { Adapter } from 'aesthetic';
import { toArray } from 'aesthetic-utils';

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

var JSSAdapter = function (_Adapter) {
  _inheritsLoose(JSSAdapter, _Adapter);

  function JSSAdapter(jss) {
    var _this;

    _this = _Adapter.call(this) || this;

    _defineProperty(_assertThisInitialized(_this), "jss", void 0);

    _defineProperty(_assertThisInitialized(_this), "keyframes", {});

    _defineProperty(_assertThisInitialized(_this), "sheets", {});

    _defineProperty(_assertThisInitialized(_this), "handleCharset", function (sheet, charset) {
      sheet.addAtRule('@charset', "\"" + charset + "\"");
    });

    _defineProperty(_assertThisInitialized(_this), "handleCss", function (css) {
      _this.getStyleSheetManager().injectStatements(css);
    });

    _defineProperty(_assertThisInitialized(_this), "handleFallback", function (ruleset, name, value) {
      ruleset.addProperty('fallbacks', [].concat(toArray(ruleset.properties.fallbacks), value.map(function (fallback) {
        var _ref;

        return _ref = {}, _ref[name] = fallback, _ref;
      })));
    });

    _defineProperty(_assertThisInitialized(_this), "handleFontFace", function (sheet, fontFaces) {
      var current = sheet.atRules['@font-face'] || [];
      sheet.addAtRule('@font-face', [].concat(current, fontFaces));
    });

    _defineProperty(_assertThisInitialized(_this), "handleGlobal", function (sheet, selector, ruleset) {
      var current = sheet.atRules['@global'] || sheet.createSheet();
      current.addRuleset(ruleset);
      sheet.addAtRule('@global', current);
    });

    _defineProperty(_assertThisInitialized(_this), "handleImport", function (sheet, paths) {
      sheet.addAtRule('@import', paths);
    });

    _defineProperty(_assertThisInitialized(_this), "handleKeyframe", function (sheet, keyframe, animationName) {
      _this.keyframes[animationName] = animationName;
      sheet.addAtRule("@keyframes " + animationName, keyframe);
    });

    _defineProperty(_assertThisInitialized(_this), "handleMedia", function (ruleset, query, value) {
      ruleset.addNested("@media " + query, value);
    });

    _defineProperty(_assertThisInitialized(_this), "handleNested", function (ruleset, selector, value) {
      ruleset.addNested("&" + selector, value);
    });

    _defineProperty(_assertThisInitialized(_this), "handleProperty", function (ruleset, name, value) {
      if (name === 'animationName') {
        ruleset.addProperty(name, _this.syntax.injectKeyframes(String(value), _this.keyframes).join(', '));
      } else {
        ruleset.addProperty(name, value);
      }
    });

    _defineProperty(_assertThisInitialized(_this), "handleSupport", function (ruleset, query, value) {
      ruleset.addNested("@supports " + query, value);
    });

    _defineProperty(_assertThisInitialized(_this), "handleViewport", function (sheet, ruleset) {
      sheet.addAtRule('@viewport', ruleset);
    });

    _this.jss = jss;

    _this.syntax.on('attribute', _this.handleNested).on('charset', _this.handleCharset).on('css', _this.handleCss).on('fallback', _this.handleFallback).on('font-face', _this.handleFontFace).on('global', _this.handleGlobal).on('import', _this.handleImport).on('keyframe', _this.handleKeyframe).on('media', _this.handleMedia).on('property', _this.handleProperty).on('pseudo', _this.handleNested).on('selector', _this.handleNested).on('support', _this.handleSupport).on('viewport', _this.handleViewport);

    return _this;
  }

  var _proto = JSSAdapter.prototype;

  _proto.flushStyles = function flushStyles(styleName) {
    var sheet = this.sheets[styleName];

    if (sheet) {
      sheet.attach();
    }
  };

  _proto.isParsedBlock = function isParsedBlock(block) {
    return typeof block === 'string';
  };

  _proto.parseStyleSheet = function parseStyleSheet(styleSheet, _ref2) {
    var name = _ref2.name,
        theme = _ref2.theme;
    this.sheets[name] = this.jss.createStyleSheet(styleSheet, {
      media: 'screen',
      meta: name + "-" + theme
    });
    this.sheets[name].attach();
    return this.sheets[name].classes;
  };

  _proto.purgeStyles = function purgeStyles() {
    Object.values(this.sheets).forEach(function (sheet) {
      sheet.detach();
    });
    this.sheets = {};
  };

  _proto.transformToClassName = function transformToClassName(styles) {
    return styles.join(' ');
  };

  return JSSAdapter;
}(Adapter);

/**
 * @copyright   2017-2019, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */

export default JSSAdapter;
